const apiUtils = require("api-utils");
const winUtils = require("window-utils");
const EventEmitter = require("events").EventEmitter;
var allItems = [];

exports.Item = EventEmitter.compose({
    constructor: function(options) {
		var requirements = {
			label: { is: ["string"], ok: function(v) v.length > 0 },
			onClick: { is: ["function"] }
		};
		options = apiUtils.validateOptions(options, requirements);
		this._id = "jetpack-tabs-context-menu-item" + allItems.length;
		this._label = options.label;
		this.on("click", options.onClick);
		allItems.push(this);
		for (var win in winUtils.windowIterator()) {
			addItemUI(win, this);
		}
	},
	_id: null,
	_label: null,
});

var winTrackerDelegate = {
	init: function() {
		var winTracker = new winUtils.WindowTracker(this);
		require("unload").ensure(winTracker);
	},
	onTrack: function(win) {
		allItems.forEach(function(item) { addItemUI(win, item); });
	},
	onUntrack: function(win) {
		allItems.forEach(function(item) { removeItemUI(win, item); });
	},
};

winTrackerDelegate.init();

function addItemUI(win, item) {
   if (win.location.href != "chrome://browser/content/browser.xul")
	  return;
	var elt = win.document.createElement("menuitem");
	elt.setAttribute("id", item._id);
	elt.setAttribute("label", item._label);
	elt.addEventListener("command", function() emitClickEvent(win, item), false);
	var parent = win.document.getElementById("tabContextMenu");
	parent.appendChild(elt);
}

function removeItemUI(win, item) {
   if (win.location.href != "chrome://browser/content/browser.xul")
	  return;
	var elt = win.document.getElementById(item._id);
	elt.removeEventListener("command", function() emitClickEvent(win, item), false);
	var parent = win.document.getElementById("tabContextMenu");
	parent.removeChild(elt);
}

function emitClickEvent(win, item) {
	var xulTab = win.document.popupNode;
	require("tabs");
	item._emit("click", xulTab);
}

